/*
 *
 * Copyright 2009 Andrey Kholmanskih. All rights reserved.
 */

package tutorial.jdbreport;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import tutorial.jdbreport.helper.Test;
import tutorial.jdbreport.helper.TestDao;

import jdbreport.design.model.TemplateBook;
import jdbreport.model.DetailGroup;
import jdbreport.model.ReportBook;
import jdbreport.model.io.LoadReportException;
import jdbreport.source.IterableDataSet;
import jdbreport.util.Utils;
import jdbreport.view.ReportEditor;

/**
 * @version 1.0 01.08.2009
 * @author Andrey Kholmanskih
 * 
 */
public class MinMaxRowsReport extends SampleItem {

	final Object[][] data = {
			{ "Zhenya", "Albers", "green", "Brazil", new Double(44.0),
					"strawberry" },
			{ "Mark", "Verzilina", "blue", "Curse of the Demon", new Double(3),
					"grapes" },
			{ "Maria", "Parcker", "black", "The Blues Brothers",
					new Double(2.7182818285), "raspberry" },
			{ "Lara", "Bunny", "red", "Airplane (the whole series)",
					new Double(15), "strawberry" },
			{ "Mike", "Black", "blue", "The Man Who Knew Too Much",
					new Double(13), "peach" },
			{ "Andrew", "Falls", "black", "Blade Runner (Director's Cut)",
					new Double(23), "broccoli" },
			{ "Luck", "Burov", "darkgreen", "Brazil", new Double(27),
					"asparagus" },
			{ "Lucy", "Weeler", "blue", "The Lady Vanishes", new Double(8),
					"kiwi" },
			{ "Ewan", "Spears", "yellow", "A Bug's Life", new Double(2),
					"strawberry" },
			{ "Katerina", "Koshkina", "darkred", "Reservoir Dogs",
					new Double(3), "raspberry" },
			{ "Lev", "Zykin", "brightred", "Jules et Jim", new Double(5),
					"raspberry" },
			{ "Daria", "White", "blue", "Pulp Fiction", new Double(3),
					"watermelon" },
			{ "Kristina", "Kholmanskih", "pink",
					"Blade Runner (Director's Cut)", new Double(21), "donut" },
			{ "Stefany", "Carrot", "blue", "Chusingura (1962)", new Double(8),
					"pear" },
			{ "Julia", "Petelina", "green", "My Life as a Dog", new Double(7),
					"onion" },
			{ "Jim", "Shilov", "gray", "None", new Double(13), "grapes" } };

	private static final String driverName = "org.apache.derby.jdbc.EmbeddedDriver";

	private static final String connectionURL = "jdbc:derby:testdb;create=true;unicode=true";

	private List<Test> testList;

	private TestDao dao;

	private void initTestList() {
		testList = new ArrayList<Test>();
		for (int i = 0; i < data.length; i++) {
			Test test = new Test((String) data[i][0], (String) data[i][1],
					(String) data[i][2], (String) data[i][3],
					(Double) data[i][4], (String) data[i][5]);
			testList.add(test);
		}
	}

	private void fillDb() throws SQLException {
		dao = new TestDao(driverName, connectionURL);
		dao.deleteTest();
		for (Test test : testList) {
			dao.insertTest(test);
		}
	}

	private void closeDb() throws SQLException {
		dao.getConnection().close();
		dao = null;
	}

	public static void main(String[] args) {
		MinMaxRowsReport report = new MinMaxRowsReport();
		report.run();
	}

	@Override
	public String getCaption() {
		return "Minimum and maximum of rows";
	}

	@Override
	public String getTemplate() {
		return "minmaxrowreport.jdbr";
	}

	@Override
	public void run() {
		initTestList();
		try {
			fillDb();
			closeDb();
		} catch (SQLException e) {
			Utils.showError(e);
		}
		try {
			
			// Usage in the program

			
			 TemplateBook tbook = new TemplateBook();
			 tbook.open(getTemplateURL());
			 tbook.addReportDataSet(new IterableDataSet("test2", testList));
			 List<DetailGroup> detailGroupList = tbook.getReportModel(0).getRowModel().getRootGroup().getDetailGroups();
			 detailGroupList.get(0).setMaxRowCount(10);
			 detailGroupList.get(1).setMinRowCount(20);
			 ReportEditor re = new ReportEditor();
			 ReportBook book = tbook.createReportBook(re);
			 re.setReportBook(book);
			 re.setVisible(true);
			

//			Collection<ReportDataSet> dsList = new ArrayList<ReportDataSet>();
//			dsList.add(new IterableDataSet("test2", testList));
//			jdbreport.JDBReport.showReport(getTemplateURL(), dsList);
			 
		} catch (LoadReportException e) {
			Utils.showError(e);
		}
	}

}
